#include <TFplus.h>

int main(int argc, char **argv)
{
  ros::init(argc, argv, "tfplus_ros_node");
  ros::NodeHandle nh("~");
  std::string id = "TFplus";
  std::string portName;
  int baud_rate;
  benewake::TFplus *tfplus_obj;

  nh.param("serial_port", portName, std::string("/dev/ttyUSB0"));
  nh.param("baud_rate", baud_rate, 115200);

  tfplus_obj = new benewake::TFplus(portName, baud_rate);
  ros::Publisher pub_range = nh.advertise<sensor_msgs::Range>(id, 1000, true);
  sensor_msgs::Range TFplus_range;
  TFplus_range.radiation_type = sensor_msgs::Range::INFRARED;
  TFplus_range.field_of_view = 0.0349066; // In degrees it is 2 is it's converted into radians.
  TFplus_range.min_range = 0.2;
  TFplus_range.max_range = 8;
  TFplus_range.header.frame_id = id;
  float dist = 0;
  ROS_INFO_STREAM("Start processing ...");

  while(ros::master::check() && ros::ok())
  {
    ros::spinOnce();
    dist = tfplus_obj->getDist();
    if(dist > 0 && dist < TFplus_range.max_range)
    {
      TFplus_range.range = dist;
      TFplus_range.header.stamp = ros::Time::now();
      pub_range.publish(TFplus_range);
    }
    else if(dist == -1.0)
    {
      ROS_ERROR_STREAM("Failed to read data. TFplus ros node stopped!");
      break;
    }
    else if(dist == 0.0)
    {
      ROS_ERROR_STREAM("Data validation error!");
    }
  }

  tfplus_obj->closePort();
}
